==> orbit-jetty-0.8.0-sources.jar: cloud/orbit/web/provider/TaskProvider.java <==
/*
 Copyright (C) 2016 Electronic Arts Inc.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1.  Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
 2.  Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
 3.  Neither the name of Electronic Arts, Inc. ("EA") nor the names of
     its contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY ELECTRONIC ARTS AND ITS CONTRIBUTORS "AS IS" AND ANY
 EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL ELECTRONIC ARTS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cloud.orbit.web.provider;

import cloud.orbit.concurrent.Task;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.ext.Provider;
import javax.ws.rs.ext.WriterInterceptor;
import javax.ws.rs.ext.WriterInterceptorContext;

import java.io.IOException;
import java.util.concurrent.CompletionException;

/**
 * Created by joe@bioware.com on 2016-02-16.
 */
@Provider
public class TaskProvider implements WriterInterceptor
{
    @Override
    public void aroundWriteTo(final WriterInterceptorContext context) throws IOException, WebApplicationException
    {
        final Object entity = context.getEntity();

        if(entity instanceof Task)
        {
            Object result = entity;

            try
            {
                result = ((Task<?>)entity).join();
            }
            catch(CompletionException e)
            {
                // We need to get the actual exception and rethrow it
                throw (RuntimeException) ((e.getCause() != null) ? e.getCause() : e);
            }

            context.setGenericType(null);

            // Set the actual type
            context.setType(result.getClass());

            // Replace the entity
            context.setEntity(result);
        }

        context.proceed();
    }
}

==> orbit-jetty-0.9.2-sources.jar: cloud/orbit/web/provider/TaskProvider.java <==
/*
 Copyright (C) 2016 Electronic Arts Inc.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1.  Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
 2.  Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
 3.  Neither the name of Electronic Arts, Inc. ("EA") nor the names of
     its contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY ELECTRONIC ARTS AND ITS CONTRIBUTORS "AS IS" AND ANY
 EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL ELECTRONIC ARTS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package cloud.orbit.web.provider;

import cloud.orbit.concurrent.Task;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.ext.Provider;
import javax.ws.rs.ext.WriterInterceptor;
import javax.ws.rs.ext.WriterInterceptorContext;

import java.io.IOException;
import java.util.concurrent.CompletionException;

@Provider
public class TaskProvider implements WriterInterceptor
{
    @Override
    public void aroundWriteTo(final WriterInterceptorContext context) throws IOException, WebApplicationException
    {
        final Object entity = context.getEntity();

        if(entity instanceof Task)
        {
            Object result = null;

            try
            {
                result = ((Task<?>)entity).join();
            }
            catch(CompletionException e)
            {
                // We need to get the actual exception and rethrow it
                throw (RuntimeException) ((e.getCause() != null) ? e.getCause() : e);
            }

            context.setGenericType(null);

            // Do we actually have a result?
            if(result != null)
            {
                // Set the type we got
                context.setType(result.getClass());

                // Replace the entity
                context.setEntity(result);
            }
            else
            {

                // Result is null, throw a 204
                throw new WebApplicationException(204);
            }
        }

        context.proceed();
    }
}
